////////////////////////////////////////////////////////////////////////////////
///
///		Title		: Image Proxy Interface File
///
///		Copyright (c) 2006, NHN Co., Ltd. ALL RIGHTS RESERVED
///
///		Team		: Desktop Service Development
///
////////////////////////////////////////////////////////////////////////////////
#pragma once

/**
* @ingroup 	Base UI	
* @brief 	Image Draw Interface
* @date		11/02/2006
* @author	telltale
* @remark	
*/
interface IImage
{
public:
	virtual ~IImage() {  }
	/**
	* @brief 	draw image
	* @param	hDC	handle to device context
	* @param	lpSrcRect	bounding rectangle to render the source image
	* @param	lpDestRect	bounding rectangle to render the component
	* @return	none
	*/
	virtual void DrawImage(HDC hDC, LPRECT lpSrcRect, LPRECT lpDestRect) = 0;
	/**
	* @brief 	Transparent draw image
	* @param	hDC	handle to device context
	* @param	lpSrcRect	bounding rectangle to render the source image
	* @param	lpDestRect	bounding rectangle to render the component
	* @return	TRUE if successful, otherwise FALSE.
	*/
	virtual BOOL TransparentBlt(HDC hDestDC, LPRECT lpSrcRect, LPRECT lpDestRect, UINT crTransparent = CLR_INVALID) = 0;

	/**
	* @brief 	get image width
	* @return	return value is width of image
	*/
	virtual int GetWidth() = 0;
	/**
	* @brief 	get image height
	* @return	return value is height of image
	*/
	virtual int GetHeight() = 0;

};

class ImageProxy : public IImage
{
public:
	ImageProxy(LPCTSTR id)
	{
		m_ID = id;		
		m_p = NULL;
	}
	~ImageProxy()
	{
		Free();
	}
	IImage* operator->()
	{
		LazyInitialize();
		return m_p; 
	}
	void Attach(LPCTSTR id)
	{
		Detach();
		m_ID = id;
		m_p = NULL;
	}
	void Detach()
	{
		Free();
	}
	//override
	void DrawImage(HDC hDC, LPRECT lpSrcRect, LPRECT lpDestRect)
	{
		LazyInitialize();
		if (m_p != NULL)
		{
			m_p->DrawImage(hDC, lpSrcRect, lpDestRect);
		}
	}
	BOOL TransparentBlt(HDC hDestDC, LPRECT lpSrcRect, LPRECT lpDestRect, UINT crTransparent = CLR_INVALID)
	{
		LazyInitialize();
		if (m_p != NULL)
		{
			return m_p->TransparentBlt(hDestDC, lpSrcRect, lpDestRect, crTransparent);
		}
		return FALSE;
	}
	int GetWidth()
	{
		//LazyInitialize();
		if (m_p != NULL)
		{
			return m_p->GetWidth();
		}
		return 0;
	}
	int GetHeight()
	{
		//LazyInitialize();
		if (m_p != NULL)
		{
			return m_p->GetHeight();
		}
		return 0;
	}
private:
	void LazyInitialize();
	void Free();

	CString m_ID;
	IImage* m_p;
};

#include	<atlimage.h>
/**
* @ingroup 	Base UI	
* @brief 	Bitmap Image Draw class
* @date		11/02/2006
* @author	telltale
* @remark	
*/
class CBitmapImage : public IImage
{
public:
	CBitmapImage();
	virtual ~CBitmapImage();

	/**
	* @brief 	load from resource by ID
	* @param	hResourceModule	resource handle
	* @param	uID	resource ID default type BMP
	* @return	if this function successes, return TRUE
	*/
	BOOL	Load(HINSTANCE hResourceModule, UINT uID);
	/**
	* @brief 	load from resource by ID and Type
	* @param	hResourceModule	resource handle
	* @param	uID	resource ID
	* @param	lpType	resource Type
	* @return	if this function successes, return TRUE
	*/
	BOOL	Load(HINSTANCE hResourceModule, UINT uID, LPCTSTR lpType);
	/**
	* @brief 	load from binary
	* @param	pBuf	image binary
	* @param	dwBufLen	binary length
	* @return	if this function successes, return TRUE
	*/
	BOOL	Load(PVOID pBuf, DWORD dwBufLen);
	/**
	* @brief 	load from file
	* @param	pszcFile	image file path
	* @return	if this function successes, return TRUE
	*/
	BOOL	LoadFromFile(LPCTSTR pszcFile);
	/**
	* @brief 	dispose resource
	* @return	none
	*/
	void	Dispose();

	operator HBITMAP();

//override
	void DrawImage(HDC hDC, LPRECT lpSrcRect, LPRECT lpDestRect);
	BOOL TransparentBlt(HDC hDestDC, LPRECT lpSrcRect, LPRECT lpDestRect, UINT crTransparent = CLR_INVALID);
	int GetWidth();
	int GetHeight();

protected:
	///< bitmap
	ATL::CImage	m_img;
	///< image binary data
	HGLOBAL m_hBuffer;
private:
	/**
	* @brief 	unload image
	* @return	none
	*/
	void	Unload();
};